import datetime
import jwt
from flask import current_app


def jwt_encode_auth_token(data):
    """Generates the auth token"""
    try:
        payload = {
            'exp': datetime.datetime.utcnow() + datetime.timedelta(
                days=current_app.config.get('JWT_TOKEN_EXPIRATION_DAYS'),
                seconds=current_app.config.get('JWT_TOKEN_EXPIRATION_SECONDS')
            ),
            'iss': current_app.config.get('JWT_TOKEN_ISSUER'),
            'iat': datetime.datetime.utcnow(),
            'sub': data
        }
        encoded = jwt.encode(payload,
                             current_app.config.get('JWT_PRIVATE_KEY'),
                             algorithm='RS256')
        return encoded
    except Exception as e:
        return e


def jwt_decode_auth_token(auth_token):
    """
    Decodes the auth token - :param auth_token: - :return: integer|string
    """
    try:
        payload = jwt.decode(
            auth_token,
            current_app.config.get('JWT_PUBLIC_KEY'),
            issuer=current_app.config.get('JWT_TOKEN_ISSUER'),
            algorithms='RS256'
        )
        return payload['sub']
    except jwt.InvalidIssuerError:
        return 'Invalid issuer. Please log in again.'
    except jwt.ExpiredSignatureError:
        return 'Signature expired. Please log in again.'
    except jwt.InvalidTokenError:
        return 'Invalid token. Please log in again.'
    except Exception as e:
        return 'Invalid token. Please log in again.'



import json
from functools import wraps
from flask import request, jsonify

from .lever_jwt import jwt_decode_auth_token


def authenticate(session=None):
    """

    :param f:
    :param session: a reference to redis db
    :return:
    """
    def decorator_function(f, *args, **kwargs):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            response_object = {
                'status': 'fail',
                'message': 'Provide a valid auth token.'
            }
            auth_header = request.headers.get('Authorization')
            if not auth_header:
                return jsonify(response_object), 401
            auth_token = auth_header.split(" ")[1]
            resp = jwt_decode_auth_token(auth_token)
            if isinstance(resp, str):
                response_object['message'] = resp
                return jsonify(response_object), 403
            if session:
                session_data = session.get(resp.get('id'))
                if session_data is None:
                    response_object["message"] = "Authentication invalid"
                    return jsonify(response_object), 401
                kwargs.update(dict(
                    session_data=json.loads(session.get(resp.get('id'))))
                )
            kwargs.update(dict(resp=resp))
            return f(*args, **kwargs)

        return decorated_function
    return decorator_function
